from jenkinsclient import jenkins_server
from jenkinsapi.plugins import Plugins

class Plugin(object):
    """
    Jenkins插件相关操作
    """
    def install(self, plugin_name):
        """
        安装插件
        :return:
        """
        server = jenkins_server.get_jenkins_server()
        restart_required = server.install_plugin(plugin_name)
        if restart_required:
            return '插件%s安装成功，重启后生效' % plugin_name
        else:
            return '插件%s安装成功并生效' % plugin_name

    def ls(self):
        """
        显示插件列表
        """
        server = jenkins_server.get_jenkins_server()
        plugins = server.get_plugins()
        plugins_list_content = '插件名称'.ljust(32) + '类型'.ljust(68) + '版本\n'
        plugins_list_content += '--------'.ljust(36) + '--------'.ljust(70) + '--------\n'
        for key in plugins.keys():
            plugins_list_content += key[0].ljust(36) + key[1].ljust(70) + plugins[key[0]]['version'] + '\n'
        return plugins_list_content

    # TODO: 解析update-center.json内容并搜索
    def search(self, plugin_name):
        """
        搜索插件
        """
        return '插件搜索功能尚未实现！'



import fire
from jenkinsclient.config import Config
from jenkinsclient.job import Job
from jenkinsclient.plugin import Plugin
from jenkinsclient import jenkins_server


class JenkinsClient(object):
    """
    Jenkins命令行客户端
    """
    def __init__(self):
        self.config = Config()
        self.job = Job()
        self.plugin = Plugin()

    def jobs(self):
        """
        显示任务列表
        """
        return Job().ls()

    def plugins(self):
        """
        显示插件列表
        """
        return Plugin().ls()

    def whoami(self):
        """
        显示当前用户
        """
        server = jenkins_server.get_jenkins_server()
        try:
            i = server.get_whoami()
        except Exception as e:
            return '操作失败：连接服务器失败'
        return i['fullName']


def main():
    fire.Fire(JenkinsClient)


if __name__ == '__main__':
    main()



import jenkins
from jenkinsapi.jenkins import Jenkins
import yaml
from jenkinsclient import config
from os.path import exists, isfile


def get_jenkins_server(type='python-jenkins'):
    if exists(config.CONFIG_FILE_PATH) and isfile(config.CONFIG_FILE_PATH):
        stream = open(config.CONFIG_FILE_PATH, 'r')
    else:
        print('配置文件%s不存在，请先使用config generate命令生成' % config.CONFIG_FILE_PATH)
        exit()

    config_json = yaml.load(stream, Loader=yaml.FullLoader)

    try:
        jenkins_servers = config_json['jenkins_servers']
        use = config_json['use']
    except KeyError as e:
        print('配置文件格式不正确，缺少%s字段' % str(e))
        exit()

    for server in jenkins_servers:
        if server['name'] == use:
            url = server['url']
            username = server['username']
            token = server['token']
    if type == 'python-jenkins':
        server = jenkins.Jenkins(url, username=username, password=token, timeout=30)
    else:
        server = Jenkins(url, username=username, password=token, timeout=30)
    return server



import platform

import yaml
import json
import os
import subprocess

CONFIG_TEMPLATE = """jenkins_servers:
- name: default
  url: http://localhost
  username: ming
  token: '1'
use: default
"""
CONFIG_FILE_PATH = os.environ['HOME'] + os.sep + '.jenkins_client.yaml'


class Config(object):
    """
    配置信息
    """
    def generate(self):
        """
        从模版生成配置文件
        :return:
        """
        with open(CONFIG_FILE_PATH, 'w') as config_file_object:
            config_file_object.write(CONFIG_TEMPLATE)

        system = platform.system()
        if system == 'Darwin':
            return_code = subprocess.call(['open', CONFIG_FILE_PATH])
            if return_code == 0:
                return '生成配置文件%s成功' % CONFIG_FILE_PATH
        elif system == 'Linux':
            return_code = subprocess.call(['vi', CONFIG_FILE_PATH])
            if return_code == 0:
                return '生成配置文件%s成功' % CONFIG_FILE_PATH
        elif system == 'Windows':
            return_code = os.startfile(CONFIG_FILE_PATH)
            if return_code == 0:
                return '生成配置文件%s成功' % CONFIG_FILE_PATH



from jenkins import JenkinsException
from jenkinsclient import jenkins_server

class Job(object):
    """
    Jenkins任务相关操作
    """

    def build(self, job_name, parameters=None):
        """
        构建任务
        """
        server = jenkins_server.get_jenkins_server()
        build_number = server.build_job(job_name, parameters=parameters)
        return '任务%s已触发，构建号：%d' % (job_name, build_number)

    def copy(self, from_name, to_name):
        """
        复制任务
        """
        server = jenkins_server.get_jenkins_server()
        try:
            server.copy_job(from_name, to_name)
        except Exception as e:
            return '复制任务%s到%s失败' % (from_name, to_name)
        return '复制任务%s到%s成功' % (from_name, to_name)

    def delete(self, job_name):
        """
        删除任务
        """
        server = jenkins_server.get_jenkins_server()
        try:
            server.delete_job(job_name)
        except JenkinsException as e:
            return '删除任务' + job_name + '失败'
        return '删除任务%s成功' % job_name

    def disable(self, job_name):
        """
        禁用任务
        """
        server = jenkins_server.get_jenkins_server()
        server.disable_job(job_name)
        return '禁用任务%s成功' % job_name

    def enable(self, job_name):
        """
        启用任务
        """
        server = jenkins_server.get_jenkins_server()
        server.enable_job(job_name)
        return '启用任务%s成功' % job_name

    def exists(self, job_name):
        """
        判断任务是否存在
        """
        server = jenkins_server.get_jenkins_server()
        exists = server.job_exists(job_name)
        if exists:
            return '任务%s存在' % job_name
        else:
            return '任务%s不存在' % job_name

    def info(self, job_name):
        """
        显示任务信息
        """
        server = jenkins_server.get_jenkins_server()
        job_info = server.get_job_info(job_name)
        return job_info

    def ls(self):
        """
        显示任务列表
        """
        server = jenkins_server.get_jenkins_server()
        job_list_content = '任务名称'.ljust(26) + '类型'.ljust(28) + '链接\n'
        job_list_content += '--------'.ljust(30) + '--------'.ljust(30) + '--------\n'
        all_jobs = server.get_all_jobs()
        for job in all_jobs:
            job_list_content += job['fullname'].ljust(30) + job['_class'].split('.')[-1].ljust(30) + job['url'] + '\n'
        return job_list_content

    def rename(self, from_name, to_name):
        """
        重命名任务
        """
        server = jenkins_server.get_jenkins_server()
        try:
            server.rename_job(from_name, to_name)
        except JenkinsException as e:
            return '重命名%s为%s失败，错误信息：%s' % (from_name, to_name, str(e))
        return '重命名%s为%s成功' % (from_name, to_name)



import setuptools

with open("README.md", "r") as readme:
    long_description = readme.read()

setuptools.setup(
    name='jenkinsclient',
    version='0.1.2',
    author="TangMing",
    author_email="hummerstudio@163.com",
    description="A powerful cross-platform Jenkins command-line client which supports multiple instances of Jennkins.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/hummerstudio/jenkinsclient',
    packages=setuptools.find_packages(),
    py_modules=['jenkins_client'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'fire', 'jenkinsapi', 'python-jenkins', 'PyYAML'
    ],
    entry_points='''
        [console_scripts]
        jenkins=jenkins_client:main
    '''
)